from abc import abstractmethod

from pydantic.v1 import BaseModel


class BaseAssistant(BaseModel):

    @abstractmethod
    def deploy_assistant(self):
        pass

    @abstractmethod
    def register_assistant(self):
        pass




import os
import sys

import pymongo
from dotenv import load_dotenv
from pymongo.server_api import ServerApi

from assistants.langregister.base import RegistrationClient

load_dotenv()
MONGO_PASSWORD = os.getenv("MONGO_PASSWORD")


class MongoAtlas(RegistrationClient):
    uri = "mongodb+srv://rajib76:{MONGO_PASSWORD}@cluster0.cp3rxai.mongodb.net/?retryWrites=true&w=majority".format(
        MONGO_PASSWORD=MONGO_PASSWORD)

    def get_client(self):
        try:
            client = pymongo.MongoClient(self.uri, server_api=ServerApi('1'))
            db = client.assistantstore
            collection = db["assistants"]
            return client, db, collection
            # return a friendly error if a URI error is thrown
        except pymongo.errors.ConfigurationError:
            print("An Invalid URI host error was received. Is your Atlas host name correct in your connection string?")
            sys.exit(1)

    def add_assistant(self, assistant_record):
        client, db, collection = self.get_client()
        try:
            result = collection.insert_many(assistant_record)
            return result
        except pymongo.errors.OperationFailure:
            print(
                "An authentication error was received. Are your username and password correct in your connection string?")
            sys.exit(1)

    def get_assistant(self, assistant_record):
        client, db, collection = self.get_client()
        docs = list(collection.find({"assistant_name": assistant_record["assistant_name"]}))
        assistant_id = ""
        file_id = ""
        if len(docs) > 0:
            for doc in docs:
                assistant_id = doc["assistant_id"]
                file_id = doc["file_id"]
            return True, assistant_id, file_id
        else:
            return False

    def update_assistant(self, assistant_record):
        client, db, collection = self.get_client()
        doc = collection.find_one_and_update({"assistant_name": assistant_record["assistant_name"]},
                                             {"$set": {"assistant_id": assistant_record["assistant_id"],
                                                       "file_id": assistant_record["file_id"]}}, new=True)
        if doc is not None:
            print("Assistant information updated in the database")
            print(doc)
        else:
            print("No assistant information updates")
        print("\n")

    def delete_assistant(self, assistant_record):
        pass



import openai
from openai.types.beta.assistant_create_params import ToolAssistantToolsRetrieval

from assistants.langassist.base import BaseAssistant
from assistants.langregister.mongo_db_atlas import MongoAtlas


class SummarizeAssistant(BaseAssistant):
    type: ToolAssistantToolsRetrieval = {"type": "retrieval"}
    file: str = ""
    model_name = "gpt-4-1106-preview"
    assistant_name: str = "summarize_assistant"

    INSTRUCTIONS = f"""You are a helpful summarization agent.You help summarize the content provided to you"""

    def list_assistant(self):
        assistants = openai.beta.assistants.list()
        return assistants
    def deploy_assistant(self):
        try:
            self.register_assistant()
            file = openai.files.create(
                file=open(self.file, "rb"),
                purpose='assistants'
            )
            assistant = openai.beta.assistants.create(
                name=self.assistant_name,
                instructions=self.INSTRUCTIONS,
                model=self.model_name,
                tools=[self.type],
                file_ids=[file.id]
            )
            assistant_record={"assistant_name":assistant.name,"assistant_id":assistant.id,"file_id":file.id}
            self.update_assistant(assistant_record)
            return file, assistant
        except Exception as e:
            print(e)

    def run_assistant(self,question,client=MongoAtlas()):
        assistant_record = {"assistant_name": self.assistant_name}
        _,assistant_id,file_id = client.get_assistant(assistant_record)
        thread = openai.beta.threads.create(
            messages=[
                {
                    "role": "user",
                    "content": question,
                    "file_ids": [file_id]
                }
            ]
        )

        run = openai.beta.threads.runs.create(
            thread_id=thread.id,
            assistant_id=assistant_id
        )

        _,answer = self.get_answer(run,thread)
        return answer

    def get_answer(self,run, thread):
        print("Looking for an answer to your question...")
        while run.status != "completed":
            run = openai.beta.threads.runs.retrieve(
                thread_id=thread.id,
                run_id=run.id
            )

        print("Phew, that was a lot of reading...")
        messages = openai.beta.threads.messages.list(
            thread_id=thread.id
        )

        annotations = messages.data[0].content[0].text.annotations
        message_content = messages.data[0].content[0].text.value
        return annotations, message_content

    def update_assistant(self, assistant_record, client=MongoAtlas()):
        client.update_assistant(assistant_record)

    def register_assistant(self,client = MongoAtlas()):
        duplicate_assistant = False
        assistant_record = {"assistant_name":self.assistant_name}
        try:
            duplicate_assistant,_,_ = client.get_assistant(assistant_record)
        except Exception as e:
            print(e)
        if duplicate_assistant:
            raise Exception("Assistant with same name exists")
        else:
            client.add_assistant([assistant_record])


import os

from assistants.langassist.summarize_assistant import SummarizeAssistant

from dotenv import load_dotenv

load_dotenv()
OPENAI_API_KEY = os.environ.get('OPENAI_API_KEY')

if __name__ == "__main__":
    # deploy the summarization assistant
    file = "gen_ai.pdf"
    assistant_name = "summarize_assistant"
    # sc = SummarizeAssistant(file=file,
    #                         assistant_name=assistant_name)
    # sc.deploy_assistant()

    # Run the assistant
    sc = SummarizeAssistant(assistant_name=assistant_name)
    question = "Summarize the content in 300 words. Please ensure all points are covered"
    print(sc.run_assistant(question))